// FamilyCode20Servlet.java - A demo servlet DS2450.
//
// Copyright (C) 1999-2002  Smart Software Consulting
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Smart Software Consulting
// 1688 Silverwood Court
// Danville, CA  94526-3079
// USA
//
// http://www.smartsc.com
//

import java.io.*;

import javax.servlet.*;
import javax.servlet.http.*;

import com.dalsemi.onewire.OneWireException;
import com.dalsemi.onewire.adapter.DSPortAdapter;
import com.dalsemi.onewire.adapter.OneWireIOException;
import com.dalsemi.onewire.container.OneWireContainer;
import com.dalsemi.onewire.utils.Address;
import com.dalsemi.onewire.utils.CRC16;

import com.smartsc.onewire.OneWireContainer20;

public class FamilyCode20Servlet
extends HttpServlet
{
	public void init()
	{
		debug = Boolean.valueOf( getInitParameter( "debug")).booleanValue();
	}

	public void doPost( HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException
	{
		doGet( req, res);
	}

	public void doGet( HttpServletRequest req, HttpServletResponse res)
	throws ServletException, IOException
	{
		String adapter = req.getParameter( OneWireServlet.ADAPTER);
		String port = req.getParameter( OneWireServlet.PORT);
		String ibString = req.getParameter( OneWireServlet.DEVICE);

		if( adapter  == null || port == null
		||  ibString == null || ibString.length() != 16)
		{
			throw new ServletException(
				getClass().getName() + ": Invalid parameters.");
		}

		byte[] id = Address.toByteArray( ibString);

		// Do HTTP header stuff
		res.setContentType( "text/html");

		// Get Writer
		PrintWriter pw = res.getWriter();

		// Start HTML
		outputHTMLHeader( pw, "Quad A/D Converter Servlet");

		DSPortAdapter pa = null;
		try
		{
			pa = OneWireServlet.getAdapter( adapter, port);
			OneWireContainer20 ib = new OneWireContainer20();
			ib.setupContainer( pa, id);

			// If POST request
			if( "POST".equals( req.getMethod()))
			{
				// Update device
				updateDevice( req, ib);
			}

			// Read memory
			ib.readMemory();

			sendBody( pw, req, pa, ib);
		}
		catch( Exception e)
		{
			OneWireServlet.sendAlert( pw, e);
			pw.println( "<P>Error: " + e.getMessage());
		}

		// Finish HTML
		outputHTMLEnd( pw);
	}

	public void sendBody(
		PrintWriter pw, HttpServletRequest req,
		DSPortAdapter pa, OneWireContainer20 ib)
	throws OneWireException
	{
		if( ib.isPresent())
		{
			// Print intro text
			pw.print( "<H1 ALIGN=\"CENTER\">Status of 1-Wire Device ");
			pw.print( ib.getAddressAsString());
			pw.print( "</H1>");

			// Start form
			outputFormStart( pw, getClass().getName(), "POST");

			// Start main table
			outputMainTableStart( pw);

			for( int channel = 0; channel < 4; ++channel)
			{
				// Start channel table
				outputChannelTableStart( pw, channelTags[channel]);
				// Current value
				int currentValue = ib.getCurrentValue( channel);
				outputTextRow( pw, "Current Value",
					Float.toString( ((float)currentValue)/1000000) + " Volts");
				// Input range
				outputSelectRow( pw, "Input Range", INPUT_RANGE_NAME,
					channelTags[channel], INPUT_RANGE_VALUES,
					ib.getInputRange( channel));
				// Output control
				int outputControl = ib.getOutputControl( channel);
				if( outputControl > 0)
				{
					--outputControl;
				}
				outputSelectRow( pw, "Output Control", OUTPUT_CONTROL_NAME,
					channelTags[channel], OUTPUT_CONTROL_VALUES,
					outputControl);
				// Resolution
				outputSelectRow( pw, "Resolution", RESOLUTION_NAME,
					channelTags[channel], RESOLUTION_VALUES,
					ib.getResolution( channel) - 1);
				// Alarm high
				outputCheckboxRow( pw, "Alarm High", ALARM_HIGH_NAME,
					channelTags[channel], ib.getAlarmHigh( channel));
				// Alarm high enable
				outputCheckboxRow( pw, "Enable", ALARM_HIGH_ENABLE_NAME,
					channelTags[channel],
					ib.getAlarmHighEnable( channel));
				// Alarm high value
				int alarmHighValue = ib.getAlarmHighValue( channel);
				outputTextInputRow( pw, "Value", ALARM_HIGH_VALUE_NAME,
					channelTags[channel], 6,
					Float.toString( ((float)alarmHighValue)/1000000), " Volts");
				// Alarm low
				outputCheckboxRow( pw, "Alarm Low", ALARM_LOW_NAME,
					channelTags[channel], ib.getAlarmLow( channel));
				// Alarm low enable
				outputCheckboxRow( pw, "Enable", ALARM_LOW_ENABLE_NAME,
					channelTags[channel],
					ib.getAlarmLowEnable( channel));
				// Alarm low value
				int alarmLowValue = ib.getAlarmLowValue( channel);
				outputTextInputRow( pw, "Value", ALARM_LOW_VALUE_NAME,
					channelTags[channel], 6,
					Float.toString( ((float)alarmLowValue)/1000000), " Volts");
				// Convert enable input
				outputCheckboxRow( pw, "Convert", CONVERT_NAME,
					channelTags[channel],
					req.getParameter( CONVERT_NAME + channelTags[channel])
						!= null);
				// End Channel Table
				outputChannelTableEnd( pw);
			}

			// End first row of main table
			pw.println( "</TR>");

			// POR Flag (device-specific, not channel-specific).
			outputPORFlagRow( pw, ib.getPORFlag());
			// End main table
			outputMainTableEnd( pw);

			// Output other form items
			outputSubmitButton( pw);
			outputHiddenParams( pw, ib.getAdapterName(),
				ib.getPortName(), ib.getAddressAsString());

			// End form
			outputFormEnd( pw);

			// Print closing text
			if( debug)
			{
				pw.println( "<HR>");
				pw.print( "<H1>Memory dump of Quad A/D 1-Wire Device ");
				pw.print( ib.getAddressAsString());
				pw.println( "</H1>");

				pw.print  ( "<pre><b>");
				pw.print  ( "        CRC16 ");
				pw.println( "07 06 05 04 03 02 01 00</b>");

				byte[][] memory = ib.getMemory();
				for( int page = 0; page < memory.length; ++page)
				{
					pw.print( "<b>Page " + page + ":</b> ");
					pw.println(
						toHexString( memory[page], memory[page].length-1, 0));
				}
				pw.println( "</pre>");
			}
		}
		else
		{
			pw.println( "<H1>1-Wire Device Not Found</H1>");
			pw.print( "1-Wire device ");
			pw.print( ib.getAddressAsString());
			pw.print( " is not present on adapter '");
			pw.print( pa.getAdapterName());
			pw.print( "' on port '");
			pw.print( pa.getPortName());
			pw.println( "'.");
		}
	}

	protected void updateDevice(
		HttpServletRequest req, OneWireContainer20 ib)
	throws OneWireException
	{
		String s;
		for( int channel = 0; channel < 4; ++channel)
		{
			// Input range
			s = req.getParameter(
				INPUT_RANGE_NAME + channelTags[channel]);
			if( INPUT_RANGE_VALUES[0].equals( s))
			{
				ib.setInputRange( channel, 0);
			}
			else
			{
				ib.setInputRange( channel, 1);
			}

			// Output control
			s = req.getParameter(
				OUTPUT_CONTROL_NAME + channelTags[channel]);
			if( OUTPUT_CONTROL_VALUES[0].equals( s))
			{
				ib.setOutputControl( channel, 0);
			}
			else if( OUTPUT_CONTROL_VALUES[1].equals( s))
			{
				ib.setOutputControl( channel, 2);
			}
			else
			{
				ib.setOutputControl( channel, 3);
			}

			// Resolution
			s = req.getParameter(
				RESOLUTION_NAME + channelTags[channel]);
			if( s != null)
			{
				int resolution = 8;
				// If one digit number
				if( s.charAt( 1) == ' ')
				{
					resolution = s.charAt( 0) - '0';
				}
				else
				{
					resolution = 10 + s.charAt( 1) - '0';
				}
				ib.setResolution( channel, resolution);
			}

			// Alarm high
			s = req.getParameter(
				ALARM_HIGH_NAME + channelTags[channel]);
			ib.setAlarmHigh( channel, s!=null);

			// Alarm high enable
			s = req.getParameter(
				ALARM_HIGH_ENABLE_NAME + channelTags[channel]);
			ib.setAlarmHighEnable( channel, s != null);

			// Alarm high value
			s = req.getParameter(
				ALARM_HIGH_VALUE_NAME + channelTags[channel]);
			if( s != null)
			{
				// Init uVolts to max value in case conversion fails
				int uVolts = OneWireContainer20.
					uVoltRanges[ ib.getInputRange( channel)];
				// Try to convert s to microvolts
				try
				{
					uVolts = (int)(1000000 * Float.valueOf( s).floatValue());
				}
				catch( NumberFormatException nfe) {}
				// Set value
				ib.setAlarmHighValue( channel, uVolts);
			}

			// Alarm low
			s = req.getParameter(
				ALARM_LOW_NAME + channelTags[channel]);
			ib.setAlarmLow( channel, s != null);

			// Alarm low enable
			s = req.getParameter(
				ALARM_LOW_ENABLE_NAME + channelTags[channel]);
			ib.setAlarmLowEnable( channel, s != null);

			// Alarm low value
			s = req.getParameter(
				ALARM_LOW_VALUE_NAME + channelTags[channel]);
			if( s != null)
			{
				// Init uVolts to min value in case conversion fails
				int uVolts = 0;
				// Try to convert s to microvolts
				try
				{
					uVolts = (int)(1000000 * Float.valueOf( s).floatValue());
				}
				catch( NumberFormatException nfe) {}
				// Set value
				ib.setAlarmLowValue( channel, uVolts);
			}
		}

		// POR Flag
		s = req.getParameter( POR_FLAG_NAME);
		ib.setPORFlag( s != null);

		// Write new settings
		ib.writeMemory();

		// Perform conversions
		String convertA = req.getParameter( CONVERT_NAME + channelTags[0]);
		String convertB = req.getParameter( CONVERT_NAME + channelTags[1]);
		String convertC = req.getParameter( CONVERT_NAME + channelTags[2]);
		String convertD = req.getParameter( CONVERT_NAME + channelTags[3]);
		ib.convert( convertA != null, convertB != null,
			convertC != null, convertD != null);
	}

	public static String toHexString( byte[] block, int first, int last)
	{
		int incr = 1;
		int len = last - first + 1;
		if( first > last)
		{
			incr = -1;
			len = 2 - len;
		}

		StringBuffer sb = new StringBuffer( 3 * len);

		for( int i = first;
		     (first <= i && i <= last) || (last <= i && i <= first);
		     i += incr)
		{
			if( (int)(block[i] & 0xff) < 0x10) sb.append( "0");
			sb.append( Integer.toHexString( (int)block[i] & 0xff));
			sb.append( " ");
		}
		return sb.toString().trim().toUpperCase();
	}

	// HTML helper methods
	protected void outputHTMLHeader( PrintWriter pw, String title)
	{
		pw.println( "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">");
		pw.println( "<HTML><HEAD><TITLE>");
		pw.println( title);
		pw.println( "</TITLE></HEAD><BODY>");
	}

	protected void outputFormStart(
		PrintWriter pw, String action, String method)
	{
		pw.print( "<FORM ACTION=\"");
		pw.print( action);
		pw.print( "\" METHOD=\"");
		pw.print( method);
		pw.println( "\" ENCTYPE=\"application/x-www-form-urlencoded\">");
		pw.println( "<CENTER>");
	}

	protected void outputMainTableStart( PrintWriter pw)
	{
		pw.print( "<TABLE BORDER=\"0\" ");
		pw.println( "CELLPADDING=\"0\" CELLSPACING=\"0\">");
		pw.println( "<TR>");
	}

	protected void outputChannelTableStart( PrintWriter pw, String channel)
	{
		pw.println( "<TD><TABLE BORDER=\"1\">");
		outputHeaderRow( pw, channel);
	}

	protected void outputHeaderRow( PrintWriter pw, String channel)
	{
		pw.print( "<TR><TH COLSPAN=\"2\">Channel ");
		pw.print( channel);
		pw.println( "</TH></TR>");
	}

	protected void outputTextRow(
		PrintWriter pw, String label, String value)
	{
		pw.print( "<TR><TD><P ALIGN=\"RIGHT\">");
		pw.print( label);
		pw.print( "</TD><TD>");
		pw.print( value);
		pw.println( "</TD></TR>");
	}

	protected void outputSelectRow(
		PrintWriter pw, String label, String name,
		String channel, String[] values, int selected)
	{
		pw.print( "<TR><TD><P ALIGN=\"RIGHT\">");
		pw.print( label);
		pw.print( "</TD><TD><SELECT NAME=\"");
		pw.print( name);
		pw.print( channel);
		pw.println( "\">");

		for( int i = 0; i < values.length; ++i)
		{
			pw.print( "<OPTION");
			if( i == selected)
			{
				pw.print( " SELECTED");
			}
			pw.print( ">");
			pw.print( values[i]);
			pw.println( "</OPTION>");
		}

		pw.println( "</SELECT></TD></TR>");
	}

	protected void outputCheckboxRow(
		PrintWriter pw, String label,
		String name, String channel, boolean checked)
	{
		pw.print( "<TR><TD><P ALIGN=\"RIGHT\">");
		pw.print( label);
		pw.print( "</TD><TD><INPUT TYPE=\"CHECKBOX\" NAME=\"");
		pw.print( name);
		pw.print( channel);
		pw.print( "\" VALUE=\"");
		pw.print( name);
		pw.print( channel);
		pw.print( "\"");
		if( checked)
		{
			pw.print( " CHECKED=\"CHECKED\"");
		}
		pw.println( "></TD></TR>");
	}

	protected void outputPORFlagRow( PrintWriter pw, boolean checked)
	{
		pw.print( "<TR><TD COLSPAN=\"4\"><P ALIGN=\"CENTER\"><BR>");
		pw.print( "Device Power-On Reset Flag&nbsp;");
		pw.print( "<INPUT TYPE=\"CHECKBOX\" NAME=\"");
		pw.print( POR_FLAG_NAME);
		pw.print( "\" VALUE=\"");
		pw.print( POR_FLAG_NAME);
		pw.print( "\"");
		if( checked)
		{
			pw.print( " CHECKED=\"CHECKED\"");
		}
		pw.println( "></TD></TR>");
	}

	protected void outputTextInputRow(
		PrintWriter pw, String label,
		String name, String channel, int size, String value, String units)
	{
		pw.print( "<TR><TD><P ALIGN=\"RIGHT\">");
		pw.print( label);
		pw.print( "</TD><TD><INPUT TYPE=\"TEXT\" NAME=\"");
		pw.print( name);
		pw.print( channel);
		pw.print( "\" SIZE=\"");
		pw.print( size);
		pw.print( "\" VALUE=\"");
		pw.print( value);
		pw.print( "\">");
		pw.print( units);
		pw.println( "</TD></TR>");
	}

	protected void outputChannelTableEnd( PrintWriter pw)
	{
		pw.println( "</TABLE></TD>");
	}

	protected void outputMainTableEnd( PrintWriter pw)
	{
		pw.println( "</TR></TABLE>");
	}

	protected void outputSubmitButton( PrintWriter pw)
	{
		pw.print( "<BR><INPUT TYPE=\"SUBMIT\" ");
		pw.print( "NAME=\"");
		pw.print( SUBMIT);
		pw.println( "\" VALUE=\"Update\">");
	}

	protected void outputHiddenParams(
		PrintWriter pw, String adapter, String port, String device)
	{
		pw.print( "<INPUT TYPE=\"HIDDEN\" NAME=\"");
		pw.print( OneWireServlet.ADAPTER);
		pw.print( "\" VALUE=\"");
		pw.print( adapter);
		pw.println( "\">");
		pw.print( "<INPUT TYPE=\"HIDDEN\" NAME=\"");
		pw.print( OneWireServlet.PORT);
		pw.print( "\" VALUE=\"");
		pw.print( port);
		pw.println( "\">");
		pw.print( "<INPUT TYPE=\"HIDDEN\" NAME=\"");
		pw.print( OneWireServlet.DEVICE);
		pw.print( "\" VALUE=\"");
		pw.print( device);
		pw.println( "\">");
	}

	protected void outputFormEnd( PrintWriter pw)
	{
		pw.println( "</CENTER></FORM>");
	}

	protected void outputHTMLEnd( PrintWriter pw)
	{
		pw.println( "</BODY></HTML>");
	}

	public static final String SUBMIT = "Submit";

	public static final String INPUT_RANGE_NAME       = "inputRange";
	public static final String OUTPUT_CONTROL_NAME    = "outputControlMask";
	public static final String RESOLUTION_NAME        = "resolution";
	public static final String ALARM_HIGH_NAME        = "alarmHigh";
	public static final String ALARM_HIGH_ENABLE_NAME = "alarmHighEnable";
	public static final String ALARM_HIGH_VALUE_NAME  = "alarmHighValue";
	public static final String ALARM_LOW_NAME         = "alarmLow";
	public static final String ALARM_LOW_ENABLE_NAME  = "alarmLowEnable";
	public static final String ALARM_LOW_VALUE_NAME   = "alarmLowValue";
	public static final String POR_FLAG_NAME          = "porFlag";
	public static final String CONVERT_NAME           = "convert";

	public static final String[] channelTags = { "A", "B", "C", "D" };

	public static final String[] INPUT_RANGE_VALUES =
	{
		"2.56 Volts",
		"5.12 Volts"
	};

	public static final String[] OUTPUT_CONTROL_VALUES =
	{
		"Off",
		"Low",
		"Float"
	};

	public static final String[] RESOLUTION_VALUES =
	{
		"1 bit",
		"2 bits",
		"3 bits",
		"4 bits",
		"5 bits",
		"6 bits",
		"7 bits",
		"8 bits",
		"9 bits",
		"10 bits",
		"11 bits",
		"12 bits",
		"13 bits",
		"14 bits",
		"15 bits",
		"16 bits"
	};

	private boolean debug = false;

	// For testing only!!!
	public static void main( String[] args)
	{
		byte[] data1 =
		{
			0x0a, 0x29, 0x11, 0x00, 0x00, 0x4c, 0x12,
			0x00, 0x35, 0x60, 0x00, 0x41, 0x33
		};

		System.out.println(
			Integer.toHexString( CRC16.compute( data1, 0, data1.length, 0)));

		byte[] data2 = {
			(byte)0xaa, 0x00, 0x00,
			0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
			(byte)0xdc, 0x25 };

		System.out.println(
			Integer.toHexString( CRC16.compute( data2, 0, data1.length, 0)));
	}
}
